;***********************************************************************
;       Program: k1therm.asm
;	  Bob Gotchall
;       v.f
;       rev date: 7/13/96
;       Purpose: This program is for a thermometer project.  A Motorola
;       68HC705K1 reads the temperature from a Dallas Semiconductor 1820
;       and displays the result on a simple 2-digit 7-seg LCD.  The
;       results will be Celsius or Fahrenheit depending on what state the
;       user has selected.  The user toggles F/C by resetting the K1.
;
;       The external RC network will be configured to operate at 2MHz.
;       The internal operation is 1Mz.  This is 1us per cycle.  Since
;       the 1820 has specified (with much slop) timing parameters I
;       calculated pulse durations throughout with this figure.
;
;       v.c: has translation of output to match LCD drivers
;       v.d: has CORRECT translation
;       v.e: actually disable pulldowns, so stop mode current much less
;       v.e_2: cleaned up for publication
;       v.f: count multiple stop cycles before taking measurement
;
;  Compiled with CASM05
;***********************************************************************
;
;       MC68HC05K1 Equates
;
PORTA   equ     $00     ; Port A byte address
PA0     equ     0       ; Port A bit 0
PA1     equ     1       ; Port A bit 1
PA2     equ     2       ; Port A bit 2
PA3     equ     3       ; Port A bit 3
PA4     equ     4       ; Port A bit 4
PA5     equ     5       ; Port A bit 5
PA6     equ     6       ; Port A bit 6
PA7     equ     7       ; Port A bit 7
;
PORTB   equ     $01     ; Port B byte address
PB0     equ     0       ; Port B bit 0
PB1     equ     1       ; Port B bit 1
;
DDRA    equ     $04     ; Port A Data Direction Register
;
DDRB    equ     $05     ; Port B Data Direction Register
;
TCSR    equ     $08     ; Timer Status and Control Register
RT0     equ     0       ; Real-Time Interrupt Rate Select bit 0
RT1     equ     1       ; Real-Time Interrupt Rate Select bit 1
RTIFR   equ     2       ; Real-Time Interrupt Flag Reset
TOFR    equ     3       ; Timer Overflow Flag Reset
RTIE    equ     4       ; Real-Time Interrupt Enable
TOIE    equ     5       ; Timer Overflow Interrupt Enable
RTIF    equ     6       ; Real-Time Interrupt Flag
TOF     equ     7       ; Timer Overflow Flag
;
TCR     equ     $09     ; Timer-Counter Register
;
ISCR    equ     $0A     ; Interrupt Status and Control Register
IRQR    equ     1       ; Interrupt Request Reset
IRQF    equ     3       ; External Interrupt Request Flag
IRQE    equ     7       ; External Interrupt Request Enable
;
PEBSR   equ     $0E     ; Personality EPROM Bit Select Register
;
PESCR   equ     $0F     ; Personality EPROM Status and Control Register
PEPRZF  equ     0       ; Personality EPROM Row Zero Flag
PEPGM   equ     5       ; Personality EPROM Program Control
PEDATA  equ     7       ; Personality EPROM Data
;
PDRA    equ     $10     ; Pull-Down Register Port A (Write-Only)
;
PDRB    equ     $11     ; Pull-Down Register Port B (Write-Only)
;
MOR     equ     $17     ; Mask Option Register (EPROM byte)
COPEN   equ     0       ; COP Watchdog Enable
LEVEL   equ     1       ; External Interrupt Sensitivity
PIRQ    equ     2       ; Port A Interrupt Function Enable
LVRE    equ     3       ; Low Voltage Reset Enable
SWAIT   equ     4       ; STOP conversion to WAIT
RC      equ     5       ; RC Oscillator select
PIN3    equ     6       ; Three-Pin RC Oscillator Select
SWPDI   equ     7       ; Software Pull-Down Inhibit
;
EPROG   equ     $18     ; EPROM Programming Control Register
EPGM    equ     0       ; EPROM Programming
MPGM    equ     1       ; Mask Option Register Programming
ELAT    equ     2       ; EPROM Bus Latch
;
COPR    equ     $3F0    ; COP Reset Register
;
;
RAMStart        equ     $00E0   ; Start of on chip RAM
ROMStart        equ     $0200   ; Start of on chip ROM/EPROM
ROMEnd  equ     $03EF   ; End of on chip ROM/EPROM
Vectors equ     $03F8   ; Start of Reset/Interrupt vectors


;----------equates for thermometer--------------
morbyt  equ     %10100010       ;Pulldowns inhibited, RC enabled, ext irq level enabled
negf    equ     7
hunf    equ     6
celf    equ     5

        org     RAMstart
pad     rmb     1       ; scratchpad location
flag    rmb     1       ; NHC00000 N=neg temp, H=over 100, C=celsuis
bitt    rmb     1       ; holds read/write bit (bit to be written, or bit read)
temp    rmb     1       ; holds 8 bits of temp from 1820
sign    rmb     1       ; 00 or FF denotes sign of temp. from 1820
BCDmsb  rmb     1
BCDlsb  rmb     1
hun     rmb     1
pad2    rmb     1

;***********************************************************************
;***********************************************************************
;***********************************************************************
start   org     ROMStart
        cli                     ;enable interrupts
        lda     #%00010011      ;enable RTI, 65.5ms period
        sta     tcsr
        lda     #%00100000      ;AND mask
        and     flag            ;H=0, N=0 after a reset
        sta     flag            ;C is unchanged
        lda     #%00100000      ;toggle the Celsius flag
        eor     flag            ;toggle
        sta     flag
        clr     pad             ;clear user variables
        clr     bitt
        clr     temp
        clr     sign
        clr     BCDmsb
        clr     BCDlsb
        clr     hun
; Clearing porta puts zeros on the display after reset.
        clr     porta   ; clear porta output buffer
        clr     portb   ; clear portb output buffer
        lda     #$ff
        sta     ddra    ; set porta to be output
        bset    1,portb
        lda     #%00000010      ; PB1 is output, PB0 is input
        sta     ddrb
; done initing

;$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
; Check for special cases:  if H or N flags are set, don't get new temp
; from 1820.  Instead, just display then hundreds or blank display.
lp1     brclr   negf,flag,mn1   ;if not neg skip
        bclr    negf,flag       ;clear flag for next time
        lda     #$ff            ;display illegal BCD value
        sta     porta           ; which blanks the LCD's
        jsr     waitlp
        bra     lp1

mn1     brclr   hunf,flag,mn2   ;if under 100, skip
        bclr    hunf,flag       ;clear flag for next time
        lda     #$f0            ;MSB will be blank
        ora     hun             ;MSB is blank, LSB is hundreds
        jsr     trans           ;reorder porta for LCD driver
        sta     porta
        jsr     waitlp
        bra     lp1
;neither flag was set.  Get temp and display.
mn2
lp3     jsr     reset   ; send reset pulse
        tstx            ; check for error
        bne     lp3     ; if error, try again
;good reset pulse, continue now.
good    lda     #$cc    ; "skip ROM" command
        sta     pad
        jsr     wrtbyt  ; send command to 1820
        lda     #$44    ; "convert T" command
        sta     pad
        jsr     wrtbyt  ; send command to 1820

;wait for 1820 to be done converting.
        lda     #$8     ;wait 65.5*8=524ms
waite   wait            ;conv time is 500max, 200typ
        deca
        bne     waite

        jsr     reset   ; issue another reset pulse
        lda     #$cc    ; "skip ROM" command
        sta     pad
        jsr     wrtbyt  ; send command to 1820
        lda     #$be    ; "read scratchpad" command
        sta     pad
        jsr     wrtbyt  ; send command to 1820
        jsr     rdbyt   ; get temp LSB
        lda     bitt
        sta     temp
        jsr     rdbyt   ; get temp MSB (sign byte)
        lda     bitt
        sta     sign
        jsr     reset   ; done talking to 1820
;#######################################################################
; Split now on Celsius flag.  Very different routines determine display.
; Celsius section here:
split   brset   celf,flag,Far   ;if Fahrenheit skip to Far
        tst     sign
        beq     mn4             ;if pos, skip
        bset    negf,flag       ;since neg, set flag for next time
        neg     temp            ;2's comp temp

mn4     lsr     temp            ;divide by 2
        lda     temp
        sub     #!100
        blo     mn5             ;if less than 100, skip to mn5
        bset    hunf,flag       ;set flag for next time
        ldx     #01
        stx     hun             ;hun=100
        bra     CnD             ;Go Convert and Display
mn5     add     #!100           ;add back the 100

        jmp     CnD             ;Go Convert and display

;Farenheit section here:
Far     tst     sign
        beq     mn7             ;if zero, mn7
        neg     temp            ;2's comp temp
        jsr     CtoF            ;convert the absolute value temp
        lda     temp
        sub     #!64            ;A=A-64, A=temp-64
        sta     temp
        bpl     mn8             ;means neg C temp is pos F temp
        neg     temp            ;take abs value of neg temp
        jmp     CnD             ;convert and display

mn8     bset    negf,flag       ;set neg flag for next time
        jmp     CnD

mn7     jsr     CtoF
        jmp     CnD             ;go convert and display
;**************************
;"Convert, Display, wait and loop back"
CnD     lda     temp            ;b2bcd requires argument in Acc
        jsr     b2bcd           ;convert the remainder in deg C to BCD
        lda     BCDlsb
        jsr     trans           ;reorder acc data for LCD drivers
        sta     portA           ;display on LCD
        jsr     waitlp
        jmp     lp1
;***************************
;$$$$$$$$$$$$$$$$$$$$$$$$$ End of main program $$$$$$$$$$$$$$$$$$$$$$$$$
;
;*********************** Beginning of subroutines **********************

;************************* write byte **********************************
;This subroutine uses write bit (8 times, surprised?) to write the byte 
;at location "pad".
wrtbyt  lda     pad     ;get byte
        sta     bitt    ;lsb of a is in bitt(0)
        ldx     #08     ;iterate 8 times
wrt1    bsr     write   ;bitt(0) transmitted
        asr     bitt    ;shift next bit into lsb
        decx            ;dec loop counter
        bne     wrt1    ;
        rts
;************************* read byte ***********************************
;This subroutine uses read bit (8 times, surprised?) to read a byte from
;the 1820.  The info is left in bitt.
rdbyt   ldx     #08     ;iterate 8 times
        clr     bitt    ;bitt needs to be inited
rdb     asr     bitt    ;clears msb
        bsr     read    ;bitt(7) contains new info
        decx            ;dec loop counter
        bne     rdb     ;
        rts
;***********************************************************************
;************************* DS1820 write bit ****************************
;This subroutine writes a bit to the DS1820.  The bit is assumed to be 
;in lsb of bitt.
write   sei                     ;disable rti interrupts
        bclr    0,portb         ;clear output buffer at PB0
        bset    0,ddrb          ;5c 0> data line is an output(and low)
        brclr   0,bitt,wr1      ;5c 0> if bitt(0) is low, skip next line
        bclr    0,ddrb          ;5c <1 data line is high now (since 						  ;bitt(0)=1)
wr1     lda     #!10            ;2c 10x6cyc=60cyc. 60x1us=60us
wr2     deca                    ;3c
        bne     wr2             ;3c
        bclr    0,ddrb          ;5c data is input, line rises
        cli                     ;enable rti interrupts
        rts
;************************* DS1820 read bit *****************************
;This subroutine reads a bit from the data line and stores the result in
;bitt(0)
read    sei                     ;disable rti interrupts
        bclr    0,portb         ;clear output buffer at PB0
        bset    0,ddrb          ;5c 0> data line is an output(and low)
        bclr    0,ddrb          ;5c <? data line is input
        nop                     ;2c 1us delay for Trc, line is pulling up
        nop                     ;2c
        nop                     ;2c have delayed 6us.
        lda     portb           ;3c <? sample!
        and     #%00000001      ;mask off pb1
        beq     rd1             ;if zero, skip next line
        bset    7,bitt          ;set bitt(7)
        cli                     ;enable rti interrupts
        rts
rd1     bclr    7,bitt          ;clear bitt(7)
        cli                     ;enable rti interrupts
        rts
;************************* DS1820 reset pulse **************************
;This subroutine attempts to send a reset pulse to the DS1820.  If it 
;gets a response, x=0 (no error).  If the DS1820 fails to pull the data 
;line low, the routine returns x=1.
;
reset   sei                     ;disable rti interrupts
        bset    0,ddrb          ;data is an output
        bclr    0,portb         ;5c 0> bring data line low
        lda     #!120           ;120x6=720 cyc. 720x1uS=720uS low pulse
re1     deca                    ;3c 0> start delay loop
        bne     re1             ;3c 0> not done counting.
;I am done transmitting reset pulse.  Now receive presence pulse
        bclr    0,ddrb          ;5c <1 data is input
        lda     #!10            ;6x10=60cyc. 60x1uS=60us 1820 waits
re2     deca                    ;3cyc
        bne     re2             ;3cyc
;The 1820 should be pulling down data line; sample until low.
;The window times out at 200us, return with error.
;Once the line has been sampled low, wait 480uS.
        lda     #!20                    ;20x11=220cyc.  220x1us=220us
re4     brclr   0,portb,re3             ;5cyc, if low, go to wait 480us
        deca                            ;3cyc, data not low, inc counter
        bne     re4                     ;3cyc
        ldx     #01                     ;x=1, error code
        cli                             ;enable rti interrupts
        rts                             ;return, with error
;
re3     lda     #!80            ;80x6=480cyc. 480x1us=480us
re5     deca                    ;3c
        bne     re5             ;3c
;reset pulse successful: x=0 and return
        clrx
        cli                     ;enable rti interrupts
        rts                     ;Done with reset pulse

;************************ Convert Celsius to Fahrenheit *****************
; This routine simulates the boring Celsius to Fahrenheit lookup table.  
; The input is the 2xC (twice Celsius) temperature in Temp.  The result 
; is in temp and hun.  Temp is copied in Acc.  Modifies Pad, 
; A,X,hun,temp.
CtoF    clr     hun
        lda     #!32
        sta     pad     ;pad will contain the working Far. temp
        lda     #!5
;done initing
cf5     tst     temp    ;is temp down to zero yet?
        bne     cf1     ;  if not, inc Ftemp and move on
        bra     cf6     ;  if temp IS zero, we are done.
cf1     tsta            ;if a=0, a=9 and DON'T inc Ftemp
        bne     cf3     ;if a<>0, inc Ftemp (skip to cf3)
        lda     #!9     ;  else a=9 and skip inc Ftemp
        bra     cf2     ;  skip inc of Ftemp
cf3     inc     pad     ;inc F temp
        deca            ;dec 9's counter
cf2     dec     temp    ;dec temp
;Check for F temp =100
        ldx     #!100
        cpx     pad     ;x-F temp
        bne     cf5     ;if 100 > F temp, go back
        inc     hun
        bset    hunf,flag       ;set hun flag for next time
        clr     pad     ;since F temp =100, inc hun flag and pad=0
        bra     cf5
cf6     lda     pad
        sta     temp
        rts
;***********************************************************************
; b2bcd (binary to bcd) converts the binary number in the accumulator 
; into a 3 digit BCD number in BCDlsb and BCDmsb.  Binary number is 
; assumed to be a 2's compliment signed magnitude number.  Can't be 
; greater than 127.
b2bcd   clr     BCDlsb
        clr     BCDmsb
b2      add     #%11110110      ;subtract 10 from A
        bpl     b1              ;if still positive, go to b1
        bra     swapnibb        ;a was <10.  we need to wrap up.
b1      inc     BCDlsb          ;A was >10.  inc lsb
        sta     temp            ;hold A for a while
        lda     BCDlsb          ;a=BCDlsb
        add     #%11110110      ;a=BCDlsb-10.
        bpl     b3              ;BCDlsb>0, deal with it (b3)
        lda     temp            ;put a back
        bra     b2              ;
b3      inc     BCDmsb          ;the BCD lsb was 10, inc msb
        clr     BCDlsb          ;necessary?
        lda     temp            ;put a back
        bra     b2
swapnibb        add     #!10    ;add the last 10 back
                lsl     BCDlsb
                lsl     BCDlsb
                lsl     BCDlsb
                lsl     BCDlsb
                ora     BCDlsb
                sta     BCDlsb
                rts

;************************** Translate **********************************
; Translate reorders the data in Acc A for communication with the LCD 
; drivers.  I do this to simplify the PCB design.
trans   sta     pad
        clr     pad2            ;destination of modified byte
        brclr   0,pad,tr0       ;pa0?
        bset    0,pad2          ;pa0-->pa0
tr0     brclr   1,pad,tr1       ;pa1?
        bset    2,pad2          ;pa1-->pa2
tr1     brclr   2,pad,tr2       ;pa2?
        bset    3,pad2          ;pa2-->pa3
tr2     brclr   3,pad,tr3       ;pa3?
        bset    1,pad2          ;pa3-->pa1
tr3     brclr   4,pad,tr4       ;pa4?
        bset    7,pad2          ;pa4-->pa7
tr4     brclr   5,pad,tr5       ;pa5?
        bset    5,pad2          ;pa5-->pa5
tr5     brclr   6,pad,tr6       ;pa6?
        bset    4,pad2          ;pa6-->pa4
tr6     brclr   7,pad,tr7       ;pa7?
        bset    6,pad2          ;pa7-->pa6
tr7     lda     pad2
        sta     pad             ;answer is in Acc, pad1 and 2
        rts
;************************** Translate **********************************
;***************************** Waitloop ********************************
; Waitloop doesn't wait or loop.  Instead it turns PB1 low to turn off
; the external FET and allow the cap on the IRQ line to discharge.  Then 
; the micro goes into STOP.  When the cap discharges below Vil, the 
; micro wakes up and goes to the eisr section. v.f: go through n cycles.
waitlp
        lda     #!10            ;A will count stop cycles. 10??
;turn off FET, slow discharge begins
wl1     bclr    1,portb         ;don't tristate, turn OFF the fet.
        stop                    ;stop until irq falls low
        deca
        bne     wl1             ;if <>0, loop back
        rts
;******************** External ISR *************************************
;Here, the external cap has discharged to Vil (1v for 5v operation) and 
;the k1 has awakened.  The k1 will not exit this routine until the 
;external cap has charged to Vih (3.5v for 5v operation).  Still, we 
;hope it has charged fully to 5v instead.  This is why we wait for it to 
;charge.
eisr    bset    1,portb         ;will output a 1, but is input now
        bset    1,ddrb          ;output now, charge up cap
        bset    rtie,tcsr       ;just in case, enable rt interrupts
        wait                    ;wait for cap to charge
        rti
;************************ Timer ISR ************************************
; Wait is sometimes used (during temp conversion and cap charging) just 
; as a delay.  When the mft generates it's irq, just return.
tisr    bset    rtifr,tcsr      ;clear rtif
        rti
;*********************** vectors ***************************************
vect    org     $03f8           ;start of user vectors
        fdb     tisr            ;MFT interrupt
        fdb     eisr            ;external IRQ interrupt
        fdb     start           ;unused SWI
        fdb     start           ;reset vector $03FE-03FF

        org     mor             ;program MOR
        db      morbyt




